window.TGR_TeamEditor = (function (window, document, $) {
  'use strict';
  //Document Ready
  var app = {
    debug: false,
    members: [],
    layouts: {},
  };
  var Admin;//Admin Team Grid
  var xbox;//Xbox Framework

  $(function () {
    app.debug = TGR_PUBLIC_JS.debug_mode === 'on';
    xbox = window.XBOX;
    Admin = window.AdminTeamGrid;

    app.prefix = Admin.prefix;

    app.init();
  });

  app.init = function () {
    app.$form = $('body.post-type-teamgrid_teams form[name="post"]');
    app.$post_body = $('body.post-type-teamgrid_teams #post-body');

    //Save post
    app.$post_body.on('click', '#save-team:not(.mtt-disabled)', app.submit_save_post);

    //Para generar el design se debe presionar también la tecla Ctrl
    app.$post_body.on('click', 'li.tab-item-customization', app.generate_design);
    app.$post_body.on('click', 'li.tab-item-filters', app.generate_filter_design);
    app.init_designs();
    app.init_filter_designs();

    //On click button preview team
    app.$post_body.on('click', '#tgr-btn-preview-team', function(){
      app.go_to_preview_team();
    });

  };

  app.submit_save_post = function(){
    event.preventDefault();

    var form_valid = true;
    var check_form = false;
    if (typeof app.$form[0].checkValidity === 'function') {
      check_form = true;
    }
    if (check_form) {
      form_valid = app.$form[0].checkValidity();
    }
    if (check_form && !form_valid) {
      var message = app.$form.getValidationMessages('.xbox-field input[type="date"], .xbox-field input[type="time"]');
      if (message) {
        alert(message);
      } else {
        alert('Error: Some fields are not valid.');
      }
      return;
    }

    var $btn = $(this);
    $btn.addClass('mtt-disabled').find('i').remove();
    $btn.append("<i class='xbox-icon xbox-icon-spinner xbox-icon-spin mtt-ml-8'></i>");

    $.xboxConfirm({
      title: TGR_ADMIN_JS.texts.saving_changes,
      content: TGR_ADMIN_JS.texts.please_wait,
      hide_confirm: true,
      hide_cancel: true,
      hide_close: true,
      wrap_class: 'transparent-confirm',
    });

    var save_interval = setInterval(function () {
      if (!$('#publish').hasClass('disabled')) {
        clearInterval(save_interval);

        //Save post
        $('#publish').click();
      }
    }, 300);
  };

  app.init_designs = function(){
    $('.tgr-design').each(function (index, container) {
      var design_id = $(container).data('id');
      var layout_number = $(container).data('layout');
      if( TGR_DESIGNS && TGR_DESIGNS[design_id] && TGR_DESIGNS[design_id][layout_number]){
        $(container).TeamGrid(TGR_DESIGNS[design_id][layout_number]);
      }
    });
  };

  app.init_filter_designs = function(){
    $('.tgr-filter-design').each(function (index, container) {
      var filter_id = $(container).data('id');
      if( TGR_FILTER_DESIGNS && TGR_FILTER_DESIGNS[filter_id] ){
        $(container).TeamGrid(TGR_FILTER_DESIGNS[filter_id]);
      }
    });
  };


  app.generate_design = function (e) {
    //Para generar el design se debe presionar también la tecla Ctrl
    if( !Admin.isCmdKey(e) ){
      return;
    }
    var options = jQuery.extend(true, {}, TGR_TEAMS[Admin.get_object_id()]);
    options.id = '';
    options.refresh = true;
    options.isDesign = true;
    delete options.title;
    delete options.showTitle;
    delete options.className;
    delete options.responsiveBy;
    options.filters.show = false;
    delete options.layout.columns;
    delete options.layout.truncateDescription;
    delete options.layout.list.isRight;
    //Por ahora el design no proporciona estilos para el displayer
    options.displayer = {};
    options.displayer.type = "none";
    options.members = ["1", "2", "3", "4"];//Los miembros de prueba

    var json_options = JSON.stringify(options);
    Admin.copy_to_clipboard(json_options);
  };

  app.generate_filter_design = function (e) {
    //Para generar el design se debe presionar también la tecla Ctrl
    if( !Admin.isCmdKey(e) ){
      return;
    }

    var options = {
      id: '',
      refresh: false,
      isDesign: false,
      isFilterDesign: true,
      displayer: {
        type: 'none'
      }
    };

    var filters_options = jQuery.extend(true, {}, TGR_TEAMS[Admin.get_object_id()].filters);
    filters_options.show = true;
    filters_options.multipleChoice = false;
    filters_options.showFilterCount = false;
    //Los filtros de prueba
    filters_options.testFilters = ["Web Developer", "Designer", "IT Manager", "Mechanical", "Dentist", "Architect"];

    options.filters = filters_options;

    var json_options = JSON.stringify(options);
    Admin.copy_to_clipboard(json_options);
  };


  app.go_to_preview_team = function(duration, offset){
    duration = duration || 550;
    offset = offset || 150;
    Admin.scroll_to($('#tgr-preview-team'), duration, offset);
  };

  app.get_animation_effect_open_popup = function () {
    if(app.is_displayer_slide_in('slide-in-left')){
      return 'tgr-slideInLeft';
    } else if(app.is_displayer_slide_in('slide-in-right')){
      return 'tgr-slideInRight'
    }
    return 'tgr-zoomIn';
  };

  app.get_animation_effect_close_popup = function (in_effect) {
    var default_effect = 'tgr-zoomOut';
    if(app.is_displayer_slide_in('slide-in-left')){
      default_effect = 'tgr-slideOutLeft';
    } else if(app.is_displayer_slide_in('slide-in-right')){
      default_effect = 'tgr-slideOutRight';
    }
    if( TGR_PUBLIC_JS.animation_effects[in_effect] ){
      return TGR_PUBLIC_JS.animation_effects[in_effect];
    }
    return default_effect;
  };

  app.is_displayer_none = function () {
    return Admin.get_field_value('tgr_displayer') == 'none';
  };

  app.is_displayer_popup = function () {
    return Admin.get_field_value('tgr_displayer') == 'popup';
  };

  app.is_displayer_slide_in = function(position) {
    var displayer = Admin.get_field_value('tgr_displayer', 'midefecto');
    if (position) {
      return displayer == position;
    }
    return displayer == 'slide-in-left' || displayer == 'slide-in-right';
  };

  app.is_displayer_inline = function () {
    return Admin.get_field_value('tgr_displayer') == 'inline';
  };

  app.is_layout_table = function (value) {
    return value == '17';
  };

  app.fields = function () {
    return {
        selectMembers: app.$post_body.find('.xbox-field-id-mts_select-members'),
        members: app.$post_body.find('.xbox-field-id-mts_members'),
        sortMembers: app.$post_body.find('.xbox-field-id-mts_sort-members'),
    }
  };

  app.get_design_options = function (design_id, layout) {
    var options = TGR_DESIGNS[design_id][layout];
    return options || false;
  };

  app.get_filter_design_options = function (design_id) {
    var options = TGR_FILTER_DESIGNS[design_id];
    return options || false;
  };


  //Debug
  function c(msg) {
    console.log(msg);
  }

  function cc(msg, msg2) {
    console.log(msg, msg2);
  }

  function clog(msg, msg2) {
    if (app.debug) {
      if( msg2 ){
        console.log(msg, msg2);
      } else {
        console.log(msg);
      }
    }
  }

  return app;

})(window, document, jQuery);
