;(function (window, document, $) {
  'use strict';
  var app = {
    debug: false,
    changedCustomizationFields: false,
    updatingDefaultValues: false,
    layouts: {},
  };
  var TeamEditor;//Team Editor
  var Admin;//Admin Team Grid
  var xbox;//Xbox Framework

  //Document Ready
  $(function () {
    app.debug = TGR_PUBLIC_JS.debug_mode === 'on';
    TeamEditor = window.TGR_TeamEditor;
    Admin = window.AdminTeamGrid;
    xbox = window.XBOX;

    app.prefix = Admin.prefix;
    app.layouts = TGR_PUBLIC_JS.layouts;

    app.init();
  });

  app.init = function () {
    app.$form = $('body.post-type-teamgrid_teams form[name="post"]');
    app.$post_body = $('body.post-type-teamgrid_teams #post-body');

    app.$post_body.on('click', '.btn-use-design', app.should_use_design);
    app.$post_body.on('click', '.btn-use-filter-design', app.use_filter_design);

    app.on_change_design_layout();
    app.on_change_general_fields();
    app.on_change_displayer_fields();
    app.on_change_popup_fields();
    app.on_change_inline_fields();
    app.on_change_list_fields();
    app.on_change_table_fields();
    app.on_change_customization_general();
    app.on_change_customization_layout();
    app.on_change_customization_displayer();
    app.on_change_customization_social();
    app.on_change_filters_fields();
    app.on_change_skills_fields();
  };

  app.should_use_design = function (event) {
    var design_id = $(this).data('design');
    var layout = Admin.get_field_value(design_id);
    $.xboxConfirm({
      title: TGR_ADMIN_JS.texts.confirm,
      content: TGR_ADMIN_JS.texts.use_design_content,
      confirm_class: 'xbox-btn-blue',
      confirm_text: XBOX_JS.text.popup.accept_button,
      cancel_text: XBOX_JS.text.popup.cancel_button,
      onConfirm: function () {
        app.use_design(design_id, layout);
      }
    });
  };

  app.use_design = function (design_id, layout) {
    var options = TeamEditor.get_design_options(design_id, layout);
    if( !options ){
      return;
    }

    delete options.layout.grid;
    delete options.layout.list.borderBottomOnly;
    delete options.layout.table.columns;
    delete options.filters.show;
    delete options.filters.multipleChoice;
    delete options.filters.showFilterCount;
    var layout_field_values = app.get_field_values_by_options('layout', options.layout);
    var filters_field_values = app.get_field_values_by_options('filters', options.filters);
    var social_field_values = app.get_field_values_by_options('social', options.social);
    var skills_field_values = app.get_field_values_by_options('skills', options.skills);
    //console.log("layout_field_values", layout_field_values);
    //console.log("filters_field_values", filters_field_values);
    //console.log("social_field_values", social_field_values);
    //console.log("skills_field_values", skills_field_values);

    app.updatingDefaultValues = true;
    Admin.set_field_values(layout_field_values);
    Admin.set_field_values(filters_field_values);
    Admin.set_field_values(social_field_values);
    Admin.set_field_values(skills_field_values);
    app.updatingDefaultValues = false;

    app.refresh_team({
      layout: options.layout
    }, 'using design');
    TeamEditor.go_to_preview_team(800);
  };

  app.use_filter_design = function (e) {
    var design_id = $(this).data('design');
    var options = TeamEditor.get_filter_design_options(design_id);
    if( !options ){
      return;
    }

    delete options.filters.multipleChoice;
    delete options.filters.showFilterCount;
    var field_values = app.get_field_values_by_options('filters', options.filters);

    app.updatingDefaultValues = true;
    Admin.set_field_values(field_values);
    app.updatingDefaultValues = false;

    app.refresh_team({
      layout: options.layout
    }, 'using filter design');

    TeamEditor.go_to_preview_team(800);
  };

  app.get_field_values_by_options = function (key, options) {
    var field_values = [];

    for( var property in options ){
      var value = options[property];
      if( Admin.isObject(value) ){
        var prop_key = Admin.camelToUnderscore(property);
        for( var prop in value ){
          var val = value[prop];
          if( Admin.isArray(val) ){
            val = val.join(',');
          }
          field_values.push({
            name: prop_key + '-' + Admin.camelToUnderscore(prop),
            value: val === true ? 'on': (val === false ? 'off' : val),
          });
        }
      } else {
        if( Admin.isArray(value) ){
          value = value.join(',');
        }
        field_values.push({
          name: key+'-'+Admin.camelToUnderscore(property),
          value: value === true ? 'on': (value === false ? 'off' : value),
        });
      }
    }
    return field_values;
  };

  app.on_change_design_layout = function () {
    app.$post_body.on('xbox_changed_value', '.xbox-field.design-layout .xbox-element', function (event, value) {
      var layout = value;
      var design_id = $(this).data('design');
      // console.log("layout", value);
      // console.log("design_id", $(this).data('design'));
      var options = TeamEditor.get_design_options(design_id, layout);
      if( options ) {
        var $team_design = $('.tgr-team-'+design_id);
        $team_design.fadeOut(50, function(){
          $team_design.show();
          $team_design.TeamGrid(options);
        });
      }
    });
  };

  app.on_change_general_fields = function () {
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-number .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          number: value,
          type: app.layouts[value]
        },
        filters: {
          inEffect: TeamEditor.is_layout_table(value) ? 'tgr-fadeIn' : 'tgr-zoomIn',
          outEffect: TeamEditor.is_layout_table(value) ? 'tgr-fadeOut' : 'tgr-zoomOut',
        }
      }, 'tgr_layout-number');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-columns .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          columns: parseInt(value)
        }
      }, 'tgr_layout-columns');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-columns-gap .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          columnsGap: parseInt(value)
        }
      }, 'tgr_layout-columns-gap');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-columns-mobile .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          columnsMobile: parseInt(value)
        }
      }, 'tgr_layout-columns-mobile');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_members .xbox-element', function (event, value) {
      app.refresh_team({
        members: value.split(','),
      }, 'tgr_members');
    });

    //Additional Settings
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_show-title .xbox-element', function (event, value) {
      app.refresh_team({
        showTitle: value == 'on'
      }, 'tgr_show-title');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_team-description .xbox-element', function (event, value) {
      app.refresh_team({
        teamDescription: value
      }, 'tgr_team-description');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_responsive-by .xbox-element', function (event, value) {
      app.refresh_team({
        responsiveBy: value
      }, 'tgr_responsive-by');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_team-class .xbox-element', function (event, value) {
      app.refresh_team({
        className: value
      }, 'tgr_team-class');
    });
  };

  app.on_change_displayer_fields = function () {
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer .xbox-element', function (event, value) {
      var open_animation = TeamEditor.get_animation_effect_open_popup();
      var close_animation = TeamEditor.get_animation_effect_close_popup(open_animation);
      app.refresh_team({
        displayer: {
          type: value,
          popup: {
            open: {
              animation: open_animation
            },
            close: {
              animation: close_animation
            },
          }
        },
      }, 'tgr_displayer');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer-vertical-padding .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          verticalPadding: parseInt(value)
        }
      }, 'tgr_displayer-vertical-padding');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer-horizontal-padding .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          horizontalPadding: parseInt(value)
        }
      }, 'tgr_displayer-horizontal-padding');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer-target-link .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          targetLink: value
        }
      }, 'tgr_displayer-target-link');
    });
  };

  app.on_change_popup_fields = function () {
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_popup-width .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          popup: {
            width: parseInt(value)
          },
        }
      }, 'tgr_popup-width');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_popup-height .xbox-element', function (event, value) {
      //value = Admin.css.number(value, 'px');retorna 123px
      app.refresh_team({
        displayer: {
          popup: {
            height: parseInt(value)
          },
        }
      }, 'tgr_popup-height');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_popup-border-radius .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          popup: {
            borderRadius: parseInt(value)
          },
        }
      }, 'tgr_popup-border-radius');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_popup-next-item-effect .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          popup: {
            nextItemEffect: value
          }
        }
      }, 'tgr_popup-next-item-effect');
    });
  };

  app.on_change_slide_fields = function () {
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_slide-width .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          slide: {
            width: parseInt(value)
          }
        }
      }, 'tgr_slide-width');
    });
  };

  app.on_change_inline_fields = function () {
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_inline-height .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          inline: {
            height: parseInt(value)
          }
        }
      }, 'tgr_inline-height');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_inline-margin-top .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          inline: {
            marginTop: parseInt(value)
          }
        }
      }, 'tgr_inline-margin-top');
    });
  };

  app.on_change_list_fields = function () {
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_list-is-right .xbox-element', function (event, value) {
      console.log("tgr_list-is-right", value);
      app.refresh_team({
        layout: {
          list: {
            isRight: value == 'on'
          }
        }
      }, 'tgr_list-is-right');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_list-border-bottom-only .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          list: {
            borderBottomOnly: value == 'on'
          }
        }
      }, 'tgr_list-border-bottom-only');
    });


  };

  app.on_change_table_fields = function () {
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_table-bordered .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          table: {
            bordered: value == 'on'
          }
        }
      }, 'tgr_table-bordered');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_table-striped .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          table: {
            striped: value == 'on'
          }
        }
      }, 'tgr_table-striped');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_table-striped-color .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          table: {
            stripedColor: value
          }
        }
      }, 'table-striped-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_table-hover .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          table: {
            hover: value == 'on'
          }
        }
      }, 'tgr_table-hover');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_table-hover-color .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          table: {
            hoverColor: value
          }
        }
      }, 'tgr_table-hover-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_table-columns .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          table: {
            columns: value.split(',')
          }
        }
      }, 'tgr_table-columns');
    });
  };


  app.on_change_customization_general = function () {
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_team-bg-color .xbox-element', function (event, value) {
      app.refresh_team({
        bgColor: value
      }, 'tgr_team-bg-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_font-family-content .xbox-element', function (event, value) {
      app.refresh_team({
        font: {
          content: value
        },
      }, 'tgr_font-family-content');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_font-family-heading1 .xbox-element', function (event, value) {
      app.refresh_team({
        font: {
          heading1: value
        },
      }, 'tgr_font-family-heading1');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_font-family-heading2 .xbox-element', function (event, value) {
      app.refresh_team({
        font: {
          heading2: value
        },
      }, 'tgr_font-family-heading2');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-truncate-description .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          truncateDescription: parseInt(value)
        }
      }, 'tgr_layout-truncate-description');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_member-title-after-subtitle .xbox-element', function (event, value) {
      app.refresh_team({
        member:{
          titleAfterRole: value == 'on'
        }
      }, 'tgr_member-title-after-subtitle');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_show-member-headings .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          grid: {
            headings: {
              visible: value == 'on'
            }
          }
        }
      }, 'tgr_show-member-headings');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_grid-social-icons-visible .xbox-element', function (event, value) {
      app.refresh_team({
        layout:{
          grid: {
            socialIcons: {
              visible: value == 'on'
            }
          }
        }
      }, 'tgr_grid-social-icons-visible');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_grid-description-visible .xbox-element', function (event, value) {
      app.refresh_team({
        layout:{
          grid: {
            description: {
              visible: value == 'on'
            }
          }
        }
      }, 'tgr_grid-description-visible');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_grid-description-visible-hover .xbox-element', function (event, value) {
      app.refresh_team({
        layout:{
          grid: {
            description: {
              visibleOnHover: value == 'on'
            }
          }
        }
      }, 'tgr_grid-description-visible-hover');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-hide-image .xbox-element', function (event, value) {
      app.refresh_team({
        layout:{
          hideImage: value == 'on'
        }
      }, 'tgr_layout-hide-image');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-image-zoom-scale .xbox-element', function (event, value) {
      app.refresh_team({
        layout:{
          imageZoomScale: parseFloat(value)
        }
      }, 'tgr_layout-image-zoom-scale');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-image-zoom-duration .xbox-element', function (event, value) {
      app.refresh_team({
        layout:{
          imageZoomDuration: parseFloat(value)
        }
      }, 'tgr_layout-image-zoom-duration');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-image-zoom-effect .xbox-element', function (event, value) {
      app.refresh_team({
        layout:{
          imageZoomEffect: value
        }
      }, 'tgr_layout-image-zoom-effect');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-equal-heights .xbox-element', function (event, value) {
      app.refresh_team({
        layout:{
          equalHeights: value == 'on'
        }
      }, 'tgr_layout-equal-heights');
    });
  };

  app.on_change_customization_layout = function () {
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-text-size .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          textSize: value
        },
      }, 'tgr_layout-text-size');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-text-size-heading1 .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          textSizeHeading1: value
        },
      }, 'tgr_layout-text-size-heading1');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-text-size-heading2 .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          textSizeHeading2: value
        },
      }, 'tgr_layout-text-size-heading2');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-text-color .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          textColor: value
        }
      }, 'tgr_layout-text-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-text-color2 .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          textColor2: value
        }
      }, 'layout-text-color2');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-link-color .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          linkColor: value
        }
      }, 'layout-link-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-bg-color .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          bgColor: value
        }
      }, 'tgr_layout-bg-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-bg-gradient-enabled .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          bgGradientEnabled: value == 'on'
        }
      }, 'tgr_layout-bg-gradient-enabled');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-bg-gradient-color .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          bgGradientColor: value
        }
      }, 'tgr_layout-bg-gradient-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-bg-gradient-angle .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          bgGradientAngle: Admin.css.number(value, 'deg'),
        }
      }, 'tgr_layout-bg-gradient-angle');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-bg-color2 .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          bgColor2: value
        }
      }, 'tgr_layout-bg-color2');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-bg-gradient-enabled2 .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          bgGradientEnabled2: value == 'on'
        }
      }, 'tgr_layout-bg-gradient-enabled2');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-bg-gradient-color2 .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          bgGradientColor2: value
        }
      }, 'tgr_layout-bg-gradient-color2');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-bg-gradient-angle2 .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          bgGradientAngle2: Admin.css.number(value, 'deg'),
        }
      }, 'tgr_layout-bg-gradient-angle2');
    });


    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-border-width .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          borderWidth: parseInt(value)
        }
      }, 'tgr_layout-border-width');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-border-color .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          borderColor: value
        }
      }, 'tgr_layout-border-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-border-different-hover .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          borderDifferentHover: value == 'on'
        }
      }, 'tgr_layout-border-different-hover');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-border-width2 .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          borderWidth2: parseInt(value)
        }
      }, 'tgr_layout-border-width2');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_layout-border-color2 .xbox-element', function (event, value) {
      app.refresh_team({
        layout: {
          borderColor2: value
        }
      }, 'tgr_layout-border-color2');
    });
  };

  app.on_change_customization_displayer = function () {
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer-text-size .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          textSize: value
        },
      }, 'tgr_displayer-text-size');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer-text-size-heading1 .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          textSizeHeading1: value
        },
      }, 'tgr_displayer-text-size-heading1');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer-text-size-heading2 .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          textSizeHeading2: value
        },
      }, 'tgr_displayer-text-size-heading2');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer-text-color .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          textColor: value
        }
      }, 'tgr_displayer-text-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer-link-color .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          linkColor: value
        }
      }, 'tgr_displayer-link-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer-bg-color .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          bgColor: value
        }
      }, 'tgr_displayer-bg-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer-border-width .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          borderWidth: parseInt(value)
        }
      }, 'tgr_displayer-border-width');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer-border-color .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          borderColor: value
        }
      }, 'tgr_displayer-border-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer-triangle-color .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          triangleColor: value
        }
      }, 'tgr_displayer-triangle-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_displayer-constrols-color .xbox-element', function (event, value) {
      app.refresh_team({
        displayer: {
          controlsColor: value
        }
      }, 'tgr_displayer-constrols-color');
    });


  };

  app.on_change_customization_social = function () {
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_social-icons-color .xbox-element', function (event, value) {
      app.refresh_team({
        social: {
          iconsColor: value
        }
      }, 'tgr_social-icons-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_social-icons-color-hover .xbox-element', function (event, value) {
      app.refresh_team({
        social: {
          iconsColorHover: value
        }
      }, 'tgr_social-icons-color-hover');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_social-icons-border-radius .xbox-element', function (event, value) {
      app.refresh_team({
        social: {
          iconsBorderRadius: parseInt(value)
        }
      }, 'tgr_social-icons-border-radius');
    });
  };

  app.on_change_filters_fields = function(){

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-show .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          show: value == 'on'
        },
      }, 'tgr_filters-show');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-selected .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          selected: value.split(',')
        },
      }, 'tgr_filters-selected');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-sort .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          sort: value
        },
      }, 'tgr_filters-sort');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-hide-all .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          hideAll: value == 'on'
        },
      }, 'tgr_filters-hide-all');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-multiple-choice .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          multipleChoice: value == 'on'
        },
      }, 'tgr_filters-multiple-choice');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-show-filter-count .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          showFilterCount: value == 'on'
        },
      }, 'tgr_filters-show-filter-count');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-dropdown .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          dropdown: value == 'on'
        },
      }, 'tgr_filters-dropdown');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-gap .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          gap: parseInt(value)
        },
      }, 'tgr_filters-gap');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-text-color .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          textColor: value
        },
      }, 'tgr_filters-text-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-font-family .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          fontFamily: value
        },
      }, 'tgr_filters-font-family');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-uppercase-text .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          uppercaseText: value == 'on'
        },
      }, 'tgr_filters-uppercase-text');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-text-size .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          textSize: parseInt(value)
        },
      }, 'tgr_filters-text-size');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-border-width .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          borderWidth: parseInt(value)
        },
      }, 'tgr_filters-border-width');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-border-radius .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          borderRadius: parseInt(value)
        },
      }, 'tgr_filters-border-radius');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-padding .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          padding: value
        },
      }, 'tgr_filters-padding');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-border-color .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          borderColor: value
        },
      }, 'tgr_filters-border-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-bg-color .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          bgColor: value
        },
      }, 'tgr_filters-bg-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-bg-gradient-enabled .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          bgGradientEnabled: value == 'on'
        },
      }, 'tgr_filters-bg-gradient-enabled');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-bg-gradient-color .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          bgGradientColor: value
        },
      }, 'tgr_filters-bg-gradient-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-bg-gradient-angle .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          bgGradientAngle: Admin.css.number(value, 'deg')
        },
      }, 'tgr_filters-bg-gradient-angle');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-text-color-active .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          textColorActive: value
        },
      }, 'tgr_filters-text-color-active');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-border-width-active .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          borderWidthActive: parseInt(value)
        },
      }, 'tgr_filters-border-width-active');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-border-color-active .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          borderColorActive: value
        },
      }, 'tgr_filters-border-color-active');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-border-bottom-only-active .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          borderBottomOnlyActive: value == 'on'
        },
      }, 'tgr_filters-border-bottom-only-active');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-bg-color-active .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          bgColorActive: value
        },
      }, 'tgr_filters-bg-color-active');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-bg-gradient-enabled-active .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          bgGradientEnabledActive: value == 'on'
        },
      }, 'tgr_filters-bg-gradient-enabled-active');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-bg-gradient-color-active .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          bgGradientColorActive: value
        },
      }, 'tgr_filters-bg-gradient-color-active');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-bg-gradient-angle-active .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          bgGradientAngleActive: Admin.css.number(value, 'deg')
        },
      }, 'tgr_filters-bg-gradient-angle-active');
    });

    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-text-color-hover .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          textColorHover: value
        },
      }, 'tgr_filters-text-color-hover');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-border-color-hover .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          borderColorHover: value
        },
      }, 'tgr_filters-border-color-hover');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-bg-color-hover .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          bgColorHover: value
        },
      }, 'tgr_filters-bg-color-hover');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-bg-gradient-enabled-hover .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          bgGradientEnabledHover: value == 'on'
        },
      }, 'tgr_filters-bg-gradient-enabled-hover');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-bg-gradient-color-hover .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          bgGradientColorHover: value
        },
      }, 'tgr_filters-bg-gradient-color-hover');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_filters-bg-gradient-angle-hover .xbox-element', function (event, value) {
      app.refresh_team({
        filters: {
          bgGradientAngleHover: Admin.css.number(value, 'deg')
        },
      }, 'tgr_filters-bg-gradient-angle-hover');
    });
  };

  app.on_change_skills_fields = function(){
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_skills-bg-color .xbox-element', function (event, value) {
      app.refresh_team({
        skills: {
          bgColor: value
        },
      }, 'tgr_skills-bg-color');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_skills-bg-percent .xbox-element', function (event, value) {
      app.refresh_team({
        skills: {
          bgPercent: value
        },
      }, 'tgr_skills-bg-percent');
    });
    app.$post_body.on('xbox_changed_value', '.xbox-field-id-tgr_skills-show-percent .xbox-element', function (event, value) {
      app.refresh_team({
        skills: {
          showPercent: value == 'on'
        },
      }, 'tgr_skills-show-percent');
    });
  };

  app.refresh_team = function (options, field_name) {
    var new_options = app.update_team_options(options);
    //console.log(">>> Actualizando campo: ", field_name, options);
    //console.log("app.updatingDefaultValues", app.updatingDefaultValues);
    if( ! app.updatingDefaultValues ){
      $('#tgr-preview-team').find('.tgr-team').TeamGrid(new_options);
    }
  };

  app.update_team_options = function (options) {
    var old_options = $.extend(true, {}, TGR_TEAMS[Admin.get_object_id()] );
    //console.log("old_options", old_options);

    //Limpiar la lista de miembros previa para que se reemplazen todos
    if( options.members ){
      old_options.members = [];
    }
    //Limpiar la lista de columnas previa para que se reemplazen todos
    if( options.layout && options.layout.table && options.layout.table.columns  ){
      old_options.layout.table.columns = [];
    }

    var new_options = $.extend(true, {}, old_options, options );
    //console.log("new options", new_options);
    TGR_TEAMS[Admin.get_object_id()] = new_options;

    return new_options;
  };

  //Debug
  function c(msg) {
    console.log(msg);
  }

  function cc(msg, msg2) {
    console.log(msg, msg2);
  }

  function clog(msg, msg2) {
    if (app.debug) {
      if( msg2 ){
        console.log(msg, msg2);
      } else {
        console.log(msg);
      }
    }
  }


  return app;

})(window, document, jQuery);