<?php namespace TeamGrid\Includes;

/*
|---------------------------------------------------------------------------------------------------
| HTTP Client for WordPress
| Autor: Max López
| Version: 1.0
|---------------------------------------------------------------------------------------------------
*/

class IronDev {
    public $api_endpoint = '';
    public $response = array();
    protected $error_message = '';
    protected $has_error = false;
    public $options = array(
        'timeout' => 5,
        'headers' => array(),
        'cookies' => array(),
        'body' => null,
        'sslverify' => true,
    );

    /*
    |---------------------------------------------------------------------------------------------------
    | Contructor
    |---------------------------------------------------------------------------------------------------
    */
    public function __construct( $api_endpoint = '' ){
        $this->api_endpoint = trailingslashit( $api_endpoint );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Set value to an option
    |---------------------------------------------------------------------------------------------------
    */
    public function set_option( $option, $value ){
        $this->options[$option] = $value;
        return true;
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | GET
    |---------------------------------------------------------------------------------------------------
    */
    public function get( $url, $args = array() ){
        return $this->make_request( 'get', $url, $args );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | POST
    |---------------------------------------------------------------------------------------------------
    */
    public function post( $url, $args = array() ){
        return $this->make_request( 'post', $url, $args );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Defines whether or not there was an error in the last request
    |---------------------------------------------------------------------------------------------------
    */
    public function success(){
        return ! $this->has_error;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns the error of a request
    |---------------------------------------------------------------------------------------------------
    */
    public function get_error(){
        return $this->error_message;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns the response code
    |---------------------------------------------------------------------------------------------------
    */
    public function get_response_code(){
        return wp_remote_retrieve_response_code( $this->response );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Make the request
    |---------------------------------------------------------------------------------------------------
    */
    private function make_request( $method, $url, $args = array() ){
        $url = $this->make_url( $url );
        if( ! $url ){
            return false;
        }
        switch( $method ){
            case 'get':
                if( ! empty( $args ) ){
                    $url = $url . '?' . http_build_query( $args );
                }
                $this->response = wp_remote_get( $url, $this->options );
                break;
            case 'post':
                $this->options['body'] = (array) $args;
                $this->response = wp_remote_post( $url, $this->options );
                break;
        }

        $this->check_errors();

        return wp_remote_retrieve_body( $this->response );
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | Check for errors
    |---------------------------------------------------------------------------------------------------
    */
    private function check_errors(){
        $this->has_error = false;
        $this->error_message = '';

        if( is_wp_error( $this->response ) ){
            $this->has_error = true;
            $this->error_message = $this->response->get_error_message();
            return false;
        }
        return true;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build the url for the new query
    |---------------------------------------------------------------------------------------------------
    */
    private function make_url( $url = '' ){
        if( filter_var( $url, FILTER_VALIDATE_URL ) !== false ){
            return $url;
        } else if( filter_var( $this->api_endpoint, FILTER_VALIDATE_URL ) !== false ){
            return $this->api_endpoint . $url;
        } else{
            $this->has_error = true;
            $this->error_message = esc_html__( 'The url provided is not a valid url', 'teamgrid' );
        }
        return false;
    }

}

?>
