<?php namespace TeamGrid\Includes;


class MemberEditor {
    protected static $instance = null;
    public $plugin;

    public $path = '';

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    private function __construct( $plugin ){
        $this->plugin = $plugin;
        $this->path = trailingslashit( TGR_OPTIONS_DIR . 'member-editor' );

        $this->hooks();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone(){
    }//Stopping Clonning of Object

    public function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin = null ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Hooks
    |---------------------------------------------------------------------------------------------------
    */
    private function hooks(){
        add_action( 'xbox_init', array( $this, 'create_metabox' ), TGR_XBOX_INIT_PRIORITY );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Create metabox
    |---------------------------------------------------------------------------------------------------
    */
    public function create_metabox( $hook ){
        $save_button_name = 'publish';
        $save_button_text = esc_html__( 'Publish', 'teamgrid' );
        if( Functions::is_post_page( 'edit' ) ){
            $save_button_name = 'save';
            $save_button_text = esc_html__( 'Update', 'teamgrid' );
        }

        $options = array(
            'id' => $this->plugin->arg( 'xbox_ids', 'member-editor' ),
            'title' => esc_html__( 'Member Editor', 'teamgrid' ),
            'post_types' => array( $this->plugin->post_types['members'] ),
            'fields_prefix' => TGR_PREFIX,
            'class' => 'teamgrid',
            'header' => array(
                'icon' => '',
                'desc' => '',
                'submit-buttons-sticky' => true,
            ),
            'form_options' => array(
                'show_save_button' => true,
                'save_button_id' => 'save-member',
                'save_button_name' => $save_button_name,
                'save_button_text' => $save_button_text . '<i class="xbox-icon xbox-icon-save"></i>',
                'save_button_class' => '',
                'insert_before_buttons' => ''
            ),
            'import_settings' => array(),
            'data_' => get_option( base64_decode( 'dGdyLXBsdWdpbi1zdGF0dXM=' ) ),
            'footer' => $this->plugin->arg( 'name' ) . ' v' . TGR_VERSION,
        );
        $xbox = xbox_new_metabox( $options );
        $this->add_options( $xbox );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add options
    |---------------------------------------------------------------------------------------------------
    */
    private function add_options( $xbox ){
        $id_main_tab = 'main-tab';
        $items_main_tab = array(
            'general' => '<i class="xbox-icon xbox-icon-drivers-license"></i>' . esc_html__( 'General', 'teamgrid' ),
            'icons' => '<i class="xbox-icon xbox-icon-facebook-square"></i>' . esc_html__( 'Social Icons', 'teamgrid' ),
            'fields' => '<i class="xbox-icon xbox-icon-list-alt"></i>' . esc_html__( 'Fields', 'teamgrid' ),
            'skills' => '<i class="xbox-icon xbox-icon-magic"></i>' . esc_html__( 'Skills', 'teamgrid' ),
            'shortcodes' => '<i class="xbox-icon xbox-icon-usb"></i>' . esc_html__( 'Shortcodes', 'teamgrid' ),
        );
        $items_main_tab = apply_filters( 'teamgrid_member_editor_tab_items', $items_main_tab, $id_main_tab );

        $xbox->add_main_tab( array(
            'name' => 'Main tab',
            'id' => $id_main_tab,
            'items' => $items_main_tab,
        ) );

        $xbox->open_tab_item( 'general' );
        if( ! Settings::plugin_status() ){
            $xbox->add_field( array(
                'type' => 'html',
                'content' => Settings::plugin_status_message( $this->plugin->settings_url ),
                'options' => array(
                    'show_name' => false,
                )
            ) );
        } else{
            include $this->path . 'general.php';
        }
        $xbox->close_tab_item( 'general' );

        $xbox->open_tab_item( 'icons' );
        include $this->path . 'member-icons.php';
        $xbox->close_tab_item( 'icons' );

        $xbox->open_tab_item( 'fields' );
        include $this->path . 'member-fields.php';
        $xbox->close_tab_item( 'fields' );

        $xbox->open_tab_item( 'skills' );
        include $this->path . 'member-skills.php';
        $xbox->close_tab_item( 'skills' );

        $xbox->open_tab_item( 'shortcodes' );
        include $this->path . 'member-shortcodes.php';
        $xbox->close_tab_item( 'shortcodes' );

        $xbox->close_tab( 'main-tab' );
    }

}