<?php namespace TeamGrid\Includes;

class MemberOptions extends BaseOptions {
    protected $xbox_id = '';
    protected $is_metabox = true;
    protected $is_settings_page = false;

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    protected function __construct( $options = array() ){
        $this->xbox_id = $this->plugin->arg( 'xbox_ids', 'member-editor' );
        parent::__construct( $options );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Sets the options saved in the database or their default values
    |---------------------------------------------------------------------------------------------------
    */
    protected function set_options( $initial_options = array() ){
        $default_options = array(
            TGR_PREFIX . 'id' => $this->id,
            TGR_PREFIX . 'status' => 'on',
            TGR_PREFIX . 'type' => 'modal',
        );
        if( isset( $initial_options['index'] ) ){
            $this->index = $initial_options['index'];
        }

        //Set prefix
        foreach( $initial_options as $key => $val ){
            $initial_options[TGR_PREFIX . $key] = $val;
            unset( $initial_options[$key] );
        }

        $default_options = wp_parse_args( $initial_options, $default_options );
        $default_options = wp_parse_args( $this->general_options(), $default_options );
        $default_options = wp_parse_args( $this->fields_options(), $default_options );
        $default_options = wp_parse_args( $this->links_options(), $default_options );

        $this->options = $default_options;
        $this->status = $this->option( 'status' );

        $this->options = apply_filters( 'tgr_member_options', $this->options, $this );

        return $this->options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns the options for the js plugin
    |---------------------------------------------------------------------------------------------------
    */
    public function js_options(){
        $options = array(
            'id' => $this->id,
            'title' => $this->title,
            'permalink' => $this->permalink,
            'custom_url' =>  $this->option( 'custom-url' ),
            'image' => $this->get_image(),
            'filters' => $this->filters_string,
            'role' => $this->option( 'role' ),
            'description' => $this->get_member_description( true ),
            'content' => $this->get_member_content(),
            'date' => $this->publish_date,
            'thumbnail' => $this->thumbnail,
            'has_thumbnail' => $this->has_thumbnail,
            'social' => $this->get_social(),
            'fields' => $this->get_member_fields(),
            'skills' => $this->get_member_skills(),
        );
        return apply_filters( 'tgr_member_js_options', $options, $this );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns javascript variables with member options
    |---------------------------------------------------------------------------------------------------
    */
    public function get_js_options(){
        $return = '';
        $options = $this->js_options();
        $return .= "\n\n TGR_MEMBERS[$this->id] = " . json_encode( $options ) . ";";
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | General options
    |---------------------------------------------------------------------------------------------------
    */
    public function general_options(){
        $options = array();
        $this->set_option_to( $options, 'image', '' );
        $this->set_option_to( $options, 'image-resizing-name', Settings::get_value( 'img-resizing-1-name', 'teamgrid_square_image' ) );
        $this->set_option_to( $options, 'role', '' );
        $this->set_option_to( $options, 'description', '' );
        $this->set_option_to( $options, 'content', '' );
        $this->set_option_to( $options, 'custom-url', '' );
        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Fields
    |---------------------------------------------------------------------------------------------------
    */
    public function fields_options(){
        $options = array();
        $this->set_option_to( $options, 'fields', array() );
        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Links
    |---------------------------------------------------------------------------------------------------
    */
    public function links_options(){
        $options = array();
        $this->set_option_to( $options, 'links', array() );
        return $options;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get member filters string
    |---------------------------------------------------------------------------------------------------
    */
    public function get_member_filters_string( $separator = ', ' ){
        return implode( $separator, $this->filters );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_social
    |---------------------------------------------------------------------------------------------------
    */
    public function get_social(){
        $social = (array) $this->option( 'links', array() );
        $links = array();
        foreach( $social as $link ){
            $temp = array(
                'url' => $link['tgr_url'],
                'target' => '_blank',
                'icon' => $link['tgr_icon'],
                'custom_icon' => isset( $link['tgr_custom_icon'] ) ? $link['tgr_custom_icon'] : '',
                'title' => isset( $link['tgr_title'] ) ? $link['tgr_title'] : '',
            );
            $links[] = $temp;
        }
        return $links;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get member fields
    |---------------------------------------------------------------------------------------------------
    */
    public function get_member_fields(){
        $member_fields = (array) $this->option( 'fields', array() );
        $fields = array();
        foreach( $member_fields as $field ){
            //Disregard the fields that do not have an icon or name
            if( empty( $field['tgr_icon'] ) && empty( $field['tgr_name'] ) ){
                continue;
            }
            $temp = array(
                'icon' => $field['tgr_icon'],
                'name' => $field['tgr_name'],
                'value' => $field['tgr_value'],
                'url_type' => $field['tgr_url-type'],
                'url' => $field['tgr_url'],
                'target' => '_blank',
            );
            $fields[] = $temp;
        }
        return $fields;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get member field names
    |---------------------------------------------------------------------------------------------------
    */
    public function get_member_fields_string( $separator = ', ' ){
        $items = array();
        foreach( $this->get_member_fields() as $field ){
            $value = !empty( $field['value'] ) ? ': ' . $field['value'] : '';
            $items[] = $field['name'].$value;
        }
        return implode( $separator, $items );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get member skills
    |---------------------------------------------------------------------------------------------------
    */
    public function get_member_skills(){
        $member_skills = (array) $this->option( 'skills', array() );
        $skills = array();
        foreach( $member_skills as $skill ){
            //Disregard skills that have no name
            if( empty( $skill['tgr_name'] ) ){
                continue;
            }
            $temp = array(
                'name' => $skill['tgr_name'],
                'time' => $skill['tgr_time'],
                'percent' => $skill['tgr_percent'],
            );
            $skills[] = $temp;
        }
        return $skills;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get member skill names
    |---------------------------------------------------------------------------------------------------
    */
    public function get_member_skills_string( $separator = ', '){
        $items = array();
        foreach( $this->get_member_skills() as $skill ){
            $percent = ! empty( $skill['percent'] ) ? ' - '. $skill['percent'].'%' : '';
            $items[] = $skill['name'] . $percent;
        }
        return implode( $separator, $items );
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | get_member_content
    |---------------------------------------------------------------------------------------------------
    */
    public function get_member_content(){
        if( $this->option( 'content' ) ){
            return Functions::parse_content( $this->option( 'content' ) );
        }
        return '';
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_member_description
    |---------------------------------------------------------------------------------------------------
    */
    public function get_member_description( $idescription_for_js_options = false ){
        $desc = trim( $this->option( 'description' ) );
        if( ! $idescription_for_js_options ){
            $truncate_desc = (int) $this->team->option( 'layout-truncate-description' );
            if( $truncate_desc == 0 ){
                return '';
            }
            if( $truncate_desc >= 1 ){
                $desc = Functions::truncate( $desc, $truncate_desc, array(
                    'ending' => '...', 'exact' => true, 'html' => true
                ) );
            }
            $desc = wp_kses_post( $desc );
            if( stripos( $desc, '<p>' ) === 0 ){
                return $desc;
            }
            return "<p>$desc</p>";
        } else{
            $desc = wp_kses_post( $desc );
            if( stripos( $desc, '<p>' ) === 0 ){
                return $desc;
            }
            return "<p>$desc</p>";
        }
    }

}