<?php namespace TeamGrid\Includes;


class Members {
    protected static $instance = null;
    public $plugin;

    private static $members = array();
    public static $resources_loaded_in_footer = false;
    public static $resources_loaded_in_header = false;

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    public function __construct( $options = array() ){
        $this->plugin = Functions::get_plugin_instance();

        $this->hooks();
        $this->add_members();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone(){
    }//Stopping Clonning of Object

    public function __wakeup(){
    }//Stopping unserialize of object

    public static function init( $plugin = null ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Hooks
    |---------------------------------------------------------------------------------------------------
    */
    private function hooks(){
        add_action( 'wp_head', array( $this, 'load_resources_in_header' ), 10 );
        add_action( 'admin_head', array( $this, 'load_resources_in_header' ), 10 );
        add_action( 'wp_footer', array( $this, 'load_resources_in_footer' ), 10 );
        add_action( 'admin_footer', array( $this, 'load_resources_in_footer' ), 10 );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Agrega una instancia de Member
    |---------------------------------------------------------------------------------------------------
    */
    public static function add( $member ){
        if( is_a( $member, 'TeamGrid\Includes\Member' ) ){
            self::$members[$member->id] = $member;
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Obtiene una instancia de Member
    |---------------------------------------------------------------------------------------------------
    */
    public static function get( $member_id = 0 ){
        $plugin = Functions::get_plugin_instance();
        if( ! $plugin->is_published_member( $member_id ) ){
            return null;
        }
        if( ! isset( self::$members[$member_id] ) ){
            $member = new Member( array(
                'id' => $member_id
            ) );
            self::add( $member );
            return $member;
        }
        return self::$members[$member_id];
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Gets all created members
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_all_members(){
        return self::$members;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add all members
    |---------------------------------------------------------------------------------------------------
    */
    public function add_members(){
        //Only add if it is public or if we are editing a team
        if( ! is_admin() || Functions::is_editing_post_type( $this->plugin->post_types['teams'] ) ){
            self::get_saved_members();
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns all members in the database
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_saved_members(){
        if( empty( self::$members ) ){
            $posts = get_posts( array(
                'post_type' => TeamGrid::$args['post_types']['members'],
                'post_status' => 'publish',
                'posts_per_page' => -1
            ) );
            $members = array();
            foreach( $posts as $post ){
                $member = new Member( array(
                    'id' => $post->ID,
                    'publish_date' => $post->post_date,
                ) );
                $members[$member->id] = $member;
            }
            foreach( $members as $member ){
                self::add( $member );
            }
        }
        return self::$members;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | update_member_fields_test
    |---------------------------------------------------------------------------------------------------
    */
    public static function update_member_fields_test(){
        $member_id = 939;
        $exclude_ids = array( $member_id );
        //$include_ids = array( 8291, 8289 );//8291, 8289
        $include_ids = array( 943, 965, 976, 977, 978, 979, 980, 981, 982, 983, 984, 985, 986, 987, 988, 989, 990, 991, 992, 993, 994, 995 );//8291, 8289

        foreach( self::$members as $m_id => $member ){
            if( in_array( $m_id, $exclude_ids ) || ! in_array( $m_id, $include_ids ) ){
                continue;
            }

            //Update social Icons
            update_post_meta( $m_id, 'tgr_links', self::get_test_social_icons( $member_id ) );

            //Update fields
            update_post_meta( $m_id, 'tgr_fields', self::get_test_fields( $member_id ) );

            //Update skills
            update_post_meta( $m_id, 'tgr_skills', self::get_test_skills( $member_id ) );
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_test_social_icons
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_test_social_icons( $member_id ){
        $social_icons = get_post_meta( $member_id, 'tgr_links', true );
        //Updating Social icons
        $social_rand_indexs = array_rand( $social_icons, rand( 2, min( 4, count( $social_icons ) - 1 ) ) );
        $new_social = array();
        $i = 0;
        foreach( $social_rand_indexs as $index ){
            $i++;
            $temp = $social_icons[$index];
            $temp['tgr_links_name'] = "Social icon #$i";
            $new_social[] = $temp;
        }
        return $new_social;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_test_fields
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_test_fields( $member_id ){
        $fields = get_post_meta( $member_id, 'tgr_fields', true );
        $fields_address = array( "3210  Kelley Road", "251  Golf Course Drive", "2661  Lucy Lane", "2394  Park Avenue", "2914  Lewis Street", "4815  Railroad Street" );
        $fields_phone = array( "+1-202-555-0172", "+1-501-555-0175", "+1-404-555-0159", "+1-225-555-0115", "+1-803-555-0123", "+44 1632 960898", "+1-613-555-0118", "+61 1900 654 321" );
        $fields_colors = array( "Red", "Green", "Blue", "White", "Black", "Orange", "Purple", "Light blue", "Pink", "Yellow" );

        //Updating fields
        $rand = rand( 2, 4 );
        $fields = array_slice( $fields, 0, $rand );
        $new_fields = array();
        foreach( $fields as $index => $field ){
            if( $field['tgr_name'] == "Phone" ){
                $field['tgr_value'] = $fields_phone[array_rand( $fields_phone )];
            }
            if( $field['tgr_name'] == "Address" ){
                $field['tgr_value'] = $fields_address[array_rand( $fields_address )];
            }
            if( $field['tgr_name'] == "Favorite Color" ){
                $field['tgr_value'] = $fields_colors[array_rand( $fields_colors )];
            }
            $new_fields[] = $field;
        }
        return $new_fields;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | get_test_skills
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_test_skills( $member_id ){
        $skills_names = array( "HTML 5", "Javascript", "Marketing", "Photography", "PHP", "Projects management", "Sales", "Web design" );
        $skills_times = array( "3 years", "4 years", "5 years", "6 years", "7 years", "8 years", "9 years", "10 years" );
        $skills_percents = array( "60", "65", "70", "75", "80", "85", "90", "95", "100" );

        //Updating Skills
        $empty_skills = array(
            array(
                'tgr_skills_name' => 'Skill #1',
                'tgr_skills_type' => 'default',
                'tgr_skills_visibility' => 'visible',
            ),
            array(
                'tgr_skills_name' => 'Skill #2',
                'tgr_skills_type' => 'default',
                'tgr_skills_visibility' => 'visible',
            ),
            array(
                'tgr_skills_name' => 'Skill #3',
                'tgr_skills_type' => 'default',
                'tgr_skills_visibility' => 'visible',
            ),
        );
        $rand = rand( 2, 3 );
        $empty_skills = array_slice( $empty_skills, 0, $rand );
        $new_skills = array();
        foreach( $empty_skills as $index => $skill ){
            $skill_name_index = array_rand( $skills_names );
            $skill['tgr_name'] = $skills_names[$skill_name_index];
            unset( $skills_names[$skill_name_index] );
            $skill['tgr_time'] = $skills_times[array_rand( $skills_times )];
            $skill['tgr_percent'] = $skills_percents[array_rand( $skills_percents )];
            $new_skills[] = $skill;
        }
        return $new_skills;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Loading resources in header
    |---------------------------------------------------------------------------------------------------
    */
    public function load_resources_in_header(){
        if( self::$resources_loaded_in_header ){
            return;
        }
        self::$resources_loaded_in_header = true;


        echo "\n\n<!-- TGR:TeamGrid:Header:Members -->";
        echo "\n<script type='text/javascript'>";
        echo "\n var TGR_MEMBERS = {};";
        echo "\n</script>";
        echo "\n<!-- TGR:TeamGrid:Header:Members End -->\n\n";
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Loading resources in footer
    |---------------------------------------------------------------------------------------------------
    */
    public function load_resources_in_footer(){
        if( self::$resources_loaded_in_footer ){
            return;
        }
        self::$resources_loaded_in_footer = true;

        $member_js_options = "";
        foreach( self::$members as $member_id => $member ){
            $member_js_options .= $member->get_js_options();
        }

        echo "\n\n<!-- TGR:TeamGrid:Footer:Members -->";
        echo "\n<script type='text/javascript'>";
        echo "\n var TGR_MEMBERS = TGR_MEMBERS || {};";
        /*
         * The setting below is not sanitized. In order to be able to take advantage of this,
		 * a user would have to gain access to the database,
		 * in which case this is the least of your worries.
         */
        echo $member_js_options;

        echo "\n</script>";
        echo "\n<!-- TGR:TeamGrid:Footer:Members End -->\n\n";
    }
}