<?php namespace TeamGrid\Includes;

class TeamEditor {
    protected static $instance = null;
    public $plugin;

    public $path = '';
    public $members = array();

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    private function __construct( $plugin ){
        $this->plugin = $plugin;
        $this->path = trailingslashit( TGR_OPTIONS_DIR.'team-editor' );

        $this->hooks();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone(){
    }//Stopping Clonning of Object

    public function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $plugin = null ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Plugin hooks
    |---------------------------------------------------------------------------------------------------
    */
    private function hooks(){
        add_action( 'xbox_init', array( $this, 'create_metabox' ), TGR_XBOX_INIT_PRIORITY );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Create metabox
    |---------------------------------------------------------------------------------------------------
    */
    public function create_metabox( $hook ){
        $save_button_name = 'publish';
        $save_button_text = esc_html__( 'Publish', 'teamgrid' );
        if( Functions::is_post_page( 'edit' ) ){
            $save_button_name = 'save';
            $save_button_text = esc_html__( 'Update', 'teamgrid' );
        }

        $peview_button = "";
        $peview_button_text = esc_html__( 'Preview', 'teamgrid' );
        if( Functions::is_post_page( 'edit' ) ){
            $peview_button = "<a href='#' id='tgr-btn-preview-team' class='xbox-btn xbox-btn-teal'>$peview_button_text</a>";
        }

        $options = array(
            'id' => $this->plugin->arg( 'xbox_ids', 'team-editor'),
            'title' => esc_html__( 'Team Editor', 'teamgrid' ),
            'post_types' => array( $this->plugin->post_types['teams'] ),
            'fields_prefix' => TGR_PREFIX,
            'class' => 'teamgrid',
            'header' => array(
                'icon' => '',
                'desc' => '',
                'submit-buttons-sticky' => true,
            ),
            'form_options' => array(
                'show_save_button' => true,
                'save_button_id' => 'save-team',
                'save_button_name' => $save_button_name,
                'save_button_text' => $save_button_text . '<i class="xbox-icon xbox-icon-save"></i>',
                'save_button_class' => '',
                'insert_before_buttons' => $peview_button
            ),
            'import_settings' => array(),
            'data_' => get_option( base64_decode( 'dGdyLXBsdWdpbi1zdGF0dXM=' ) ),
            'footer' => $this->plugin->arg( 'name' ) . ' v' . TGR_VERSION,
        );
        $xbox = xbox_new_metabox( $options );

        $this->members = Members::get_saved_members();

        $this->add_fields( $xbox );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add options
    |---------------------------------------------------------------------------------------------------
    */
    private function add_fields( $xbox ){
        $id_main_tab = 'main-tab';
        $items_main_tab = array(
            'general' => '<i class="xbox-icon xbox-icon-cog"></i>'.esc_html__( 'General', 'teamgrid' ),
            'filters' => '<i class="xbox-icon xbox-icon-sliders"></i>'.esc_html__( 'Filters', 'teamgrid' ),
            'customization' => '<i class="xbox-icon xbox-icon-paint-brush"></i>'.esc_html__( 'Customization', 'teamgrid' ),
            'designs' => '<i class="xbox-icon xbox-icon-eyedropper"></i>'.esc_html__( 'Design Templates', 'teamgrid' ),
            'shortcodes' => '<i class="xbox-icon xbox-icon-usb"></i>'.esc_html__( 'Shortcodes', 'teamgrid' ),
            'advanced' => '<i class="xbox-icon xbox-icon-code"></i>'.esc_html__( 'Advanced', 'teamgrid' ),
        );
        $items_main_tab = apply_filters( 'teamgrid_team_editor_tab_items', $items_main_tab, $id_main_tab );

        $xbox->add_main_tab( array(
            'id' => $id_main_tab,
            'items' => $items_main_tab,
        ) );

        $xbox->open_tab_item( 'general' );
        if( ! Settings::plugin_status() ) {
            $xbox->add_field(array(
                'type' => 'html',
                'content' => Settings::plugin_status_message( $this->plugin->settings_url ),
                'options' => array(
                    'show_name' => false,
                )
            ));
        } else {
            include $this->path . 'general/general-tab.php';
        }
        $xbox->close_tab_item( 'general' );

        $xbox->open_tab_item( 'filters' );
        include $this->path . 'filters/filters.php';
        $xbox->close_tab_item( 'filters' );

        $xbox->open_tab_item( 'customization' );
        include $this->path . 'customization/customization.php';
        $xbox->close_tab_item( 'customization' );

        $xbox->open_tab_item( 'designs' );
        include $this->path . 'designs/designs.php';
        $xbox->close_tab_item( 'designs' );

        $xbox->open_tab_item( 'shortcodes' );
        include $this->path . 'shortcodes/shortcodes.php';
        $xbox->close_tab_item( 'shortcodes' );

        $xbox->open_tab_item( 'advanced' );
        include $this->path . 'advanced/advanced.php';
        $xbox->close_tab_item( 'advanced' );

        $xbox->close_tab( 'main-tab' );

        $xbox->add_html(array(
            'content' => $this->build_team_preview(),
        ));
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Get all members
    |---------------------------------------------------------------------------------------------------
    */
    private function get_all_members_items( $xbox ){
        $items = array();
        foreach( $this->members as $member ){
            //Properties required for the Xbox add_select_sortable field
            $items[$member->id] = array(
                'key' => $member->id,
                'name' => $member->title,
                'date' => $member->publish_date,
                'image_url' => $member->get_image(),
            );;
        }
        return $items;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Build the preview
    |---------------------------------------------------------------------------------------------------
    */
    public function build_team_preview(){
        if( ! Functions::is_editing_post_type( $this->plugin->post_types['teams'] ) ){
            return '';
        }
        $return = "";
        $return .= "<div id='tgr-preview-team'>";

        $return .= "<div class='tgr-preview-header'>";
        $return .= esc_html__( 'Live Preview', 'teamgrid' );
        $return .= "</div>";

        $return .= "<div class='tgr-preview-body'>";
        $team = Teams::get( Functions::post_id() );

        if( $team ){
            $return .= $team->build();
        } else {
            $return .= "<div class='tgr-team' data-id='0'>";
            $return .= "</div>";
        }

        $return .= "</div>";//tgr-preview-body
        $return .= "</div>";//tgr-preview-team
        return $return;
    }
}