<?php namespace TeamGrid\Includes;


class TeamGrid {
    public static $args = array();
    public $plugin;
    protected static $instance = null;
    public $plugin_loader = null;
    public $post_types = array();
    public $xbox_ids = array();
    public $settings_url = '';
    public $main_menu_item = '';

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    private function __construct( $args = array() ){
        self::$args = $args;
        $this->plugin = $this;
        $this->post_types = $this->arg( 'post_types' );
        $this->xbox_ids = $this->arg( 'xbox_ids' );
        $this->main_menu_item = 'edit.php?post_type=' . $this->arg( 'post_types', 'members' );

        $this->plugin_loader();
        $this->settings_url = Functions::post_type_url( $this->post_types['members'], 'edit', array( 'page' => $this->xbox_ids['settings'] ) );

        $this->hooks();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone(){
    }//Stopping Clonning of Object

    public function __wakeup(){
    }//Stopping unserialize of object

    public static function get_instance( $args = array() ){
        if( null === self::$instance ){
            self::$instance = new self( $args );
        }
        return self::$instance;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Plugin arguments
    |---------------------------------------------------------------------------------------------------
    */
    public function arg( $name = '', $key = '' ){
        if( isset( self::$args[$name] ) ){
            if( $key ){
                if( isset( self::$args[$name][$key] ) ){
                    return self::$args[$name][$key];
                } else{
                    return null;
                }
            }
            return self::$args[$name];
        }
        return null;
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | Plugin loader
    |---------------------------------------------------------------------------------------------------
    */
    private function plugin_loader(){
        include dirname( __FILE__ ) . '/class-autoloader.php';
        ClassAutoloader::run();
        $this->plugin_loader = PluginLoader::get_instance( $this );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Plugin hooks
    |---------------------------------------------------------------------------------------------------
    */
    private function hooks(){
        add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ) );
        add_action( 'wp_loaded', array( $this, 'load_resources' ) );
        add_action( 'init', array( $this, 'create_post_types' ), 10 );
        add_action( 'init', array( $this, 'create_filters_taxonomy' ), 10 );//From 951 it gives an error
        add_action( 'init', array( $this, 'create_fields_taxonomy' ), 10 );
        add_action( 'init', array( $this, 'create_skills_taxonomy' ), 10 );
        add_action( 'admin_menu', array( $this, 'add_submenu_pages' ), 10 );
        add_action( 'after_setup_theme', array( $this, 'add_post_thumbnails_support' ) );
        add_action( 'init', array( $this, 'add_image_sizes' ) );
        add_shortcode( 'teamgrid_members', array( $this, 'display_members' ) );
        add_shortcode( 'teamgrid_team', array( $this, 'display_team' ) );
        add_shortcode( 'teamgrid_member', array( $this, 'display_member' ) );
        add_shortcode( 'teamgrid_member_image', array( $this, 'display_member_image' ) );
        add_shortcode( 'teamgrid_member_role', array( $this, 'display_member_role' ) );
        add_shortcode( 'teamgrid_member_content', array( $this, 'display_member_content' ) );
        add_shortcode( 'teamgrid_member_social', array( $this, 'display_member_social' ) );
        add_shortcode( 'teamgrid_member_fields', array( $this, 'display_member_fields' ) );
        add_shortcode( 'teamgrid_member_skills', array( $this, 'display_member_skills' ) );
        add_filter( 'the_content', array( $this, 'add_member_shortcodes_to_content' ) );

        add_action( "xbox_after_save_fields", array( $this, 'update_member_post_content_and_thumbnail' ), 10, 4 );
        add_action( "xbox_after_save_fields", array( $this, 'add_or_remove_member_to_teams' ), 10, 4 );
        add_action( 'admin_notices', array( $this, 'create_top_bar' ), 1 );

        add_action( "post_row_actions", array( $this, 'add_duplicate_links' ), 10, 2 );
        add_filter( "manage_edit-{$this->arg('post_types', 'members')}_columns", array( $this, 'set_columns_members' ) );
        add_action( "manage_{$this->arg('post_types', 'members')}_posts_custom_column", array( $this, 'set_content_columns_members' ), 10, 2 );

        add_filter( "manage_edit-{$this->arg('post_types', 'teams')}_columns", array( $this, 'set_columns_teams' ) );
        add_action( "manage_{$this->arg('post_types', 'teams')}_posts_custom_column", array( $this, 'set_content_columns_teams' ), 10, 2 );

        add_action( 'admin_notices', array( $this, 'show_message_to_activate_plugin' ) );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Plugins loaded hook
    |---------------------------------------------------------------------------------------------------
    */
    public function plugins_loaded(){
        $plugin_rel_path = trailingslashit( plugin_basename( TGR_DIR ) );
        $loaded = load_plugin_textdomain( 'teamgrid', false, $plugin_rel_path . 'languages/' );

        if( ! $loaded ){
            load_textdomain( 'teamgrid', TGR_DIR . 'languages/teamgrid-' . get_locale() . '.mo' );
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Load Members & Teams
    |---------------------------------------------------------------------------------------------------
    */
    public function load_resources(){
        Members::init( $this );
        Teams::init( $this );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Image sizes
    |---------------------------------------------------------------------------------------------------
    */
    public function add_image_sizes(){
        if( function_exists( 'add_image_size' ) ){
            $image_sizes = Settings::get_image_sizes();
            add_image_size( $image_sizes['1']['name'], $image_sizes['1']['width'], $image_sizes['1']['height'], true );
            add_image_size( $image_sizes['2']['name'], $image_sizes['2']['width'], $image_sizes['2']['height'], true );
            add_image_size( $image_sizes['3']['name'], $image_sizes['3']['width'], $image_sizes['3']['height'], true );
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Make sure featured images are enabled
    |---------------------------------------------------------------------------------------------------
    */
    public function add_post_thumbnails_support(){
        if( ! current_theme_supports( 'post-thumbnails' ) ){
            add_theme_support( 'post-thumbnails' );
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Labels for custom post types
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_post_type_labels( $menu_name, $plural, $singular, $all_items ){
        return array(
            'singular_name' => $singular,
            'name' => $plural,
            'menu_name' => $menu_name,
            'add_new' => sprintf( esc_html__( 'New %s', 'teamgrid' ), $singular ),
            'name_admin_bar' => sprintf( '%s', $singular ) . ' (' . $menu_name . ')',
            'all_items' => $all_items,
            'add_new_item' => sprintf( esc_html__( 'Add %s', 'teamgrid' ), $singular ),
            'new_item' => sprintf( esc_html__( 'New %s', 'teamgrid' ), $singular ),
            'edit_item' => sprintf( esc_html__( 'Edit %s', 'teamgrid' ), $singular ),
            'update_item' => sprintf( esc_html__( 'Update %s', 'teamgrid' ), $singular ),
            'view_item' => sprintf( esc_html__( 'View %s', 'teamgrid' ), $singular ),
            'view_items' => sprintf( esc_html__( 'View %s', 'teamgrid' ), $plural ),
            'search_items' => sprintf( esc_html__( 'Search %s', 'teamgrid' ), $plural ),
            'not_found' => sprintf( esc_html__( 'No %s found', 'teamgrid' ), $plural ),
            'not_found_in_trash' => sprintf( esc_html__( 'No %s found in Trash', 'teamgrid' ), $plural ),
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Labels for custom post types
    |---------------------------------------------------------------------------------------------------
    */
    public function get_post_type_args( $args ){
        if( is_null( $args['show_ui'] ) ){
            $args['show_ui'] = true;

            $disable_roles = array();
            if( $this->settings ){
                $disable_roles = $this->settings->option( 'disable-user-roles', array() );
            }

            //Exclude user roles that cannot manage the custom post type
            if( is_user_logged_in() ){
                $user = wp_get_current_user();
                $role = $user->roles ? $user->roles[0] : false;
                if( in_array( $role, $disable_roles ) ){
                    $args['show_ui'] = false;
                }
            }
        }
        return array(
            'labels' => $args['labels'],
            'description' => '',
            'supports' => $args['supports'],
            'hierarchical' => false,
            'capability_type' => 'post',

            'public' => isset( $args['public'] ) ? $args['public'] : false,//To hide permalink when editing
            //Allows to be visible on the front-end. url.com/post-type/slug
            'publicly_queryable' => isset( $args['publicly_queryable'] ) ? $args['publicly_queryable'] : $args['public'],
            'exclude_from_search' => false,
            'show_in_nav_menus' => false,
            'show_ui' => $args['show_ui'],
            'show_in_menu' => isset( $args['show_in_menu'] ) ? $args['show_in_menu'] : $args['show_ui'],

            'menu_position' => isset( $args['menu_position'] ) ? $args['menu_position'] : null,
            'menu_icon' => isset( $args['menu_icon'] ) ? $args['menu_icon'] : null,
            'show_in_admin_bar' => true,
            'can_export' => true,
            'has_archive' => false,
            'rewrite' => isset( $args['rewrite'] ) ? $args['rewrite'] : true,
            'delete_with_user' => false
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Create custom post types
    |---------------------------------------------------------------------------------------------------
    */
    public function create_post_types(){
        $this->create_post_type_members();
        $this->create_post_type_teams();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Post type: Team Members
    |---------------------------------------------------------------------------------------------------
    */
    public function create_post_type_members(){
        if( post_type_exists( $this->arg( 'post_types', 'members' ) ) ){
            return;
        }

        $singular = esc_html__( 'Member', 'teamgrid' );
        $plural = esc_html__( 'Members', 'teamgrid' );
        $all_items = sprintf( esc_html__( 'All %s', 'teamgrid' ), $plural );
        $labels = $this->get_post_type_labels( $this->arg( 'name' ), $plural, $singular, $all_items );
        $args = $this->plugin->get_post_type_args( array(
            'labels' => $labels,
            'public' => true,
            'supports' => array( 'title' ),
            'show_ui' => null,
            'show_in_menu' => true,
            'menu_position' => 21,
            'menu_icon' => esc_url( TGR_URL . 'assets/admin/images/icon-plugin.png' ),
            'rewrite' => array(
                'slug' => Settings::get_value( 'members-slug' )
            )
        ) );
        register_post_type( $this->arg( 'post_types', 'members' ), $args );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Post type: Teams
    |---------------------------------------------------------------------------------------------------
    */
    public function create_post_type_teams(){
        if( post_type_exists( $this->arg( 'post_types', 'teams' ) ) ){
            return;
        }

        $singular = esc_html__( 'Team', 'teamgrid' );
        $plural = esc_html__( 'Teams', 'teamgrid' );
        $all_items = sprintf( esc_html__( 'All %s', 'teamgrid' ), $plural );

        $labels = $this->get_post_type_labels( $this->arg( 'name' ), $plural, $singular, $all_items );
        $args = $this->get_post_type_args( array(
            'labels' => $labels,
            'public' => false,
            'supports' => array( 'title' ),
            'show_ui' => null,
            'show_in_menu' => $this->plugin->main_menu_item,
            'menu_position' => null,
        ) );

        register_post_type( $this->arg( 'post_types', 'teams' ), $args );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add submenu pages
    |---------------------------------------------------------------------------------------------------
    */
    public function add_submenu_pages(){
        //Teams
        $singular = esc_html__( 'Team', 'teamgrid' );
        $page_title = sprintf( esc_html__( 'New %s', 'teamgrid' ), $singular );
        $menu_title = $page_title;
        $menu_slug = 'post-new.php?post_type=' . $this->plugin->arg( 'post_types', 'teams' );
        add_submenu_page( $this->plugin->main_menu_item, $page_title, $menu_title, 'manage_options', $menu_slug );

    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Create Filters
    |---------------------------------------------------------------------------------------------------
    */
    public function create_filters_taxonomy(){
        $labels = array(
            'menu_name' => esc_html__( 'Filters', 'teamgrid' ),
            'name' => esc_html__( 'Filters', 'teamgrid' ),
            'singular_name' => esc_html__( 'Filter', 'teamgrid' ),
            'search_items' => esc_html__( 'Search', 'teamgrid' ),
            'popular_items' => esc_html__( 'Popular', 'teamgrid' ),
            'all_items' => esc_html__( 'All', 'teamgrid' ),
            'parent_item' => null,
            'parent_item_colon' => null,
            'edit_item' => esc_html__( 'Edit', 'teamgrid' ),
            'update_item' => esc_html__( 'Update', 'teamgrid' ),
            'add_new_item' => esc_html__( 'Add New', 'teamgrid' ),
            'new_item_name' => esc_html__( 'New Filter', 'teamgrid' ),
            'separate_items_with_commas' => null,
            'add_or_remove_items' => esc_html__( 'Add or remove Filters', 'teamgrid' ),
            'choose_from_most_used' => null,
            'not_found' => esc_html__( 'No found', 'teamgrid' ),
        );
        $args = array(
            'labels' => $labels,
            'public' => true,
            'publicly_queryable' => false,
            'show_ui' => true,
            'hierarchical' => true,
            'show_admin_column' => false,
            'show_in_rest' => true,
            'show_in_menu' => 'edit.php?post_type=' . $this->arg( 'post_types', 'members' ),
        );
        register_taxonomy( $this->arg( 'taxonomies', 'filters' ), $this->arg( 'post_types', 'members' ), $args );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Create Fields
    |---------------------------------------------------------------------------------------------------
    */
    public function create_fields_taxonomy(){
        $labels = array(
            'menu_name' => esc_html__( 'Field names', 'teamgrid' ),
            'name' => esc_html__( 'Field names', 'teamgrid' ),
            'singular_name' => esc_html__( 'Field', 'teamgrid' ),
            'search_items' => esc_html__( 'Search', 'teamgrid' ),
            'popular_items' => esc_html__( 'Popular', 'teamgrid' ),
            'all_items' => esc_html__( 'All', 'teamgrid' ),
            'parent_item' => null,
            'parent_item_colon' => null,
            'edit_item' => esc_html__( 'Edit', 'teamgrid' ),
            'update_item' => esc_html__( 'Update', 'teamgrid' ),
            'add_new_item' => esc_html__( 'Add New', 'teamgrid' ),
            'new_item_name' => esc_html__( 'New Field', 'teamgrid' ),
            'separate_items_with_commas' => null,
            'add_or_remove_items' => esc_html__( 'Add or remove Fields', 'teamgrid' ),
            'choose_from_most_used' => null,
            'not_found' => esc_html__( 'Not found', 'teamgrid' ),
        );
        $args = array(
            'labels' => $labels,
            'public' => true,
            'publicly_queryable' => false,
            'show_ui' => true,
            'hierarchical' => true,
            'show_admin_column' => false,
            'show_in_rest' => true,
            'show_in_quick_edit' => false,
            'show_in_menu' => 'edit.php?post_type=' . $this->arg( 'post_types', 'members' ),
        );
        register_taxonomy( $this->arg( 'taxonomies', 'fields' ), $this->arg( 'post_types', 'members' ), $args );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Create Skills
    |---------------------------------------------------------------------------------------------------
    */
    public function create_skills_taxonomy(){
        $labels = array(
            'menu_name' => esc_html__( 'Skills', 'teamgrid' ),
            'name' => esc_html__( 'Skills', 'teamgrid' ),
            'singular_name' => esc_html__( 'Skill', 'teamgrid' ),
            'search_items' => esc_html__( 'Search', 'teamgrid' ),
            'popular_items' => esc_html__( 'Popular', 'teamgrid' ),
            'all_items' => esc_html__( 'All', 'teamgrid' ),
            'parent_item' => null,
            'parent_item_colon' => null,
            'edit_item' => esc_html__( 'Edit', 'teamgrid' ),
            'update_item' => esc_html__( 'Update', 'teamgrid' ),
            'add_new_item' => esc_html__( 'Add New', 'teamgrid' ),
            'new_item_name' => esc_html__( 'New Field', 'teamgrid' ),
            'separate_items_with_commas' => null,
            'add_or_remove_items' => esc_html__( 'Add or remove Skills', 'teamgrid' ),
            'choose_from_most_used' => null,
            'not_found' => esc_html__( 'Not found', 'teamgrid' ),
        );
        $args = array(
            'labels' => $labels,
            'public' => true,
            'publicly_queryable' => false,
            'show_ui' => true,
            'hierarchical' => true,
            'show_admin_column' => false,
            'show_in_rest' => true,
            'show_in_quick_edit' => false,
            'show_in_menu' => 'edit.php?post_type=' . $this->arg( 'post_types', 'members' ),
        );
        register_taxonomy( $this->arg( 'taxonomies', 'skills' ), $this->arg( 'post_types', 'members' ), $args );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Toolbar Menu
    |---------------------------------------------------------------------------------------------------
    */
    public function create_top_bar(){
        $return = '';

        if( ! Functions::is_admin_page_post_type( $this->arg( 'post_types', 'members' ) ) ){
            return;
        }

        $return .= "<div class='tgr-topbar'>";
        $return .= "<ul class='tgr-topbar-menu'>";
        $return .= "<li class='tgr-topbar-item'>";
        $return .= "<a href='" .esc_url( Functions::post_type_url( $this->arg( 'post_types', 'members' ) ) ) . "'><i class='xbox-icon xbox-icon-users'></i>" . esc_html__( 'All Members', 'teamgrid' ) . "</a>";
        $return .= "</li>";

        $return .= "<li class='tgr-topbar-item'>";
        $return .= "<a href='" .esc_url( Functions::post_type_url( $this->arg( 'post_types', 'members' ), 'new' ) ) . "'><i class='xbox-icon xbox-icon-user-plus'></i>" . esc_html__( 'New Member', 'teamgrid' ) . "</a>";
        $return .= "</li>";

        $return .= "<li class='tgr-topbar-item'>";
        $return .= "<a href='" .esc_url( Functions::post_type_url( $this->arg( 'post_types', 'teams' ), 'edit' ) ) . "'><i class='xbox-icon xbox-icon-address-book'></i>" . esc_html__( 'All Teams', 'teamgrid' ) . "</a>";
        $return .= "</li>";

        $return .= "<li class='tgr-topbar-item'>";
        $return .= "<a href='" .esc_url( Functions::post_type_url( $this->arg( 'post_types', 'teams' ), 'new' ) ) . "'><i class='xbox-icon xbox-icon-vcard'></i>" . esc_html__( 'New Team', 'teamgrid' ) . "</a>";
        $return .= "</li>";

        $return .= "<li class='tgr-topbar-item'>";
        $return .= "<a href='" .esc_url( Functions::post_type_url( $this->arg( 'post_types', 'members' ), 'edit', array( 'page' => $this->arg( 'xbox_ids', 'settings' ) ) ) ) . "'><i class='xbox-icon xbox-icon-cog'></i>" . sprintf( esc_html__( '%s Settings', 'teamgrid' ), 'TeamGrid' ) . "</a>";
        $return .= "</li>";
        $return .= "<li class='tgr-topbar-item'>";
        $return .= "<a href='http://teamgrid.net/documentation/' target='_blank'><i class='xbox-icon xbox-icon-file-text'></i>" . esc_html__( 'Documentation', 'teamgrid' ) . "</a>";
        $return .= "</li>";
        $return .= "</ul>";
        $return .= "</div>";
        echo wp_kses_post( $return );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Team Shortcode
    |---------------------------------------------------------------------------------------------------
    */
    public function display_team( $atts = array(), $content = null ){
        $atts = shortcode_atts( array(
            'id' => 0,
            'class' => '',
            'show_title' => '',
            'columns' => '',
            'viewer' => '',
            'responsive_by' => '',
        ), $atts );

        if( ! $this->is_published_team( $atts['id'] ) ){
            return '';
        }

        $return = '';
        $team_old = Teams::get( $atts['id'] );

        $team = new Team( array(
            'id' => $atts['id'],
            'team-class-shortcode' => $atts['class'],
            'show-title' => $atts['show_title'] ? $atts['show_title'] : $team_old->option( 'show-title' ),
            'layout-columns' => $atts['columns'] ? $atts['columns'] : $team_old->option( 'layout-columns' ),
            'displayer' => $atts['viewer'] ? $atts['viewer'] : $team_old->option( 'displayer' ),
            'responsive-by' => $atts['responsive_by'] ? $atts['responsive_by'] : $team_old->option( 'responsive-by' ),
        ) );

        if( $team ){
            $return .= $team->build();
        }
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if the shortcode attribute corresponds to a valid member
    |---------------------------------------------------------------------------------------------------
    */
    public function verify_member_shortcode_attr( $atts = array() ){
        $atts = shortcode_atts( array(
            'id' => 0,
            'class' => '',
            'verify_is_published' => '1',
        ), $atts );
        if( $atts['verify_is_published'] == '1' && ! $this->is_published_member( $atts['id'] ) ){
            return false;
        }
        return $atts;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Member Shortcode (Image, Content, Fields, Social Links and Skills)
    |---------------------------------------------------------------------------------------------------
    */
    public function display_member( $atts = array(), $content = null ){
        $atts = $this->verify_member_shortcode_attr( $atts );
        if( ! is_array( $atts ) ){
            return '';
        }

        $return = '';
        $member = Members::get( $atts['id'] );
        if( $member ){
            $return .= "<div class='teamgrid-shortcode tgr-shortcode-member " . esc_attr( $atts['class'] ) . "'>";
            $return .= $member->display();
            $return .= "</div>";//teamgrid-shortcode
        }
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Member Shortcode (Image)
    |---------------------------------------------------------------------------------------------------
    */
    public function display_member_image( $atts = array(), $content = null ){
        $atts = $this->verify_member_shortcode_attr( $atts );
        if( ! is_array( $atts ) ){
            return '';
        }

        $return = '';
        $member = Members::get( $atts['id'] );
        if( $member ){
            $return .= "<div class='teamgrid-shortcode tgr-shortcode-member-image " . esc_attr( $atts['class'] ) . "'>";
            $return .= $member->build_member_image();
            $return .= "</div>";//teamgrid-shortcode
        }
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Member Shortcode (Role)
    |---------------------------------------------------------------------------------------------------
    */
    public function display_member_role( $atts = array(), $content = null ){
        $atts = $this->verify_member_shortcode_attr( $atts );
        if( ! is_array( $atts ) ){
            return '';
        }

        $return = '';
        $member = Members::get( $atts['id'] );
        if( $member ){
            $return .= "<div class='teamgrid-shortcode tgr-shortcode-member-role " . esc_attr( $atts['class'] ) . "'>";
            $return .= $member->build_member_role();
            $return .= "</div>";//teamgrid-shortcode
        }
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Member Shortcode (Content)
    |---------------------------------------------------------------------------------------------------
    */
    public function display_member_content( $atts = array(), $content = null ){
        $atts = $this->verify_member_shortcode_attr( $atts );
        if( ! is_array( $atts ) ){
            return '';
        }

        $return = '';
        $member = Members::get( $atts['id'] );
        if( $member ){
            $return .= "<div class='teamgrid-shortcode tgr-shortcode-member-content " . esc_attr( $atts['class'] ) . "'>";
            $return .= $member->build_member_content();
            $return .= "</div>";//teamgrid-shortcode
        }
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Member Shortcode (Social Icons)
    |---------------------------------------------------------------------------------------------------
    */
    public function display_member_social( $atts = array(), $content = null ){
        $atts = $this->verify_member_shortcode_attr( $atts );
        if( ! is_array( $atts ) ){
            return '';
        }

        $return = '';
        $member = Members::get( $atts['id'] );
        if( $member ){
            $return .= "<div class='teamgrid-shortcode tgr-shortcode-member-social " . esc_attr( $atts['class'] ) . "'>";
            $return .= $member->build_member_social();
            $return .= "</div>";//teamgrid-shortcode
        }
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Member Shortcode (Fields)
    |---------------------------------------------------------------------------------------------------
    */
    public function display_member_fields( $atts = array(), $content = null ){
        $atts = $this->verify_member_shortcode_attr( $atts );
        if( ! is_array( $atts ) ){
            return '';
        }

        $return = '';
        $member = Members::get( $atts['id'] );
        if( $member ){
            $return .= "<div class='teamgrid-shortcode tgr-shortcode-member-fields " . esc_attr( $atts['class'] ) . "'>";
            $return .= $member->build_member_fields();
            $return .= "</div>";//teamgrid-shortcode
        }
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Member Shortcode (Skills)
    |---------------------------------------------------------------------------------------------------
    */
    public function display_member_skills( $atts = array(), $content = null ){
        $atts = $this->verify_member_shortcode_attr( $atts );
        if( ! is_array( $atts ) ){
            return '';
        }

        $return = '';
        $member = Members::get( $atts['id'] );
        if( $member ){
            $return .= "<div class='teamgrid-shortcode tgr-shortcode-member-skills " . esc_attr( $atts['class'] ) . "'>";
            $return .= $member->build_member_skills();
            $return .= "</div>";//teamgrid-shortcode
        }
        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add member shortcodes
    |---------------------------------------------------------------------------------------------------
    */
    public function add_member_shortcodes_to_content( $content ){
        $member_id = get_the_ID();
        $before = '';
        $after = '';

        //Remove shortcodes in the content
        $content = preg_replace( "/(?!\[(\/*)?teamgrid_member_image(.*?)\])(\[(\/*)?teamgrid_member_(.*?)\])/", '', $content );

        if( is_singular( array( $this->post_types['members'] ) ) ){
            $shortcode_role = "[teamgrid_member_role id='$member_id' verify_is_published='0']";
            $shortcode_fields = "[teamgrid_member_fields id='$member_id' verify_is_published='0']";
            $shortcode_skills = "[teamgrid_member_skills id='$member_id' verify_is_published='0']";
            $shortcode_social = "[teamgrid_member_social id='$member_id' verify_is_published='0']";

            $before = do_shortcode( $shortcode_role );
            $after .= do_shortcode( $shortcode_fields );
            $after .= do_shortcode( $shortcode_skills );
            $after .= do_shortcode( $shortcode_social );
        }

        $return = $before . $content . $after;

        return $return;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Update member content and featured image
    |---------------------------------------------------------------------------------------------------
    */
    public function update_member_post_content_and_thumbnail( $data, $object_id, $updated_fields, $xbox_instance ){
        $after_saving_member = $object_id && isset( $data['post_type'] ) && $data['post_type'] == $this->plugin->arg( 'post_types', 'members' );
        if( $after_saving_member ){
            //Funcionalidad reemplazada por: add_member_shortcodes_to_content
            if( in_array( 'tgr_content', $updated_fields ) ){
                $content = $data['tgr_content'];
//                $fields = "[teamgrid_member_fields id='$object_id' verify_is_published='0']";
//                $role = "[teamgrid_member_role id='$object_id' verify_is_published='0']";
//                $skills = "[teamgrid_member_skills id='$object_id' verify_is_published='0']";
//                $social = "[teamgrid_member_social id='$object_id' verify_is_published='0']";
//                $content .= $fields . $skills . $social;
//                $content = $role . $content;
                $post_args = array(
                    'ID' => sanitize_text_field( $object_id ),
                    'post_content' => wp_kses_post( $content )
                );
                wp_update_post( $post_args );
            }
            if( ! empty( $data['tgr_image_id'] ) ){
                set_post_thumbnail( $object_id, sanitize_text_field( $data['tgr_image_id'] ) );
            }
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add or remove member to teams
    |---------------------------------------------------------------------------------------------------
    */
    public function add_or_remove_member_to_teams( $data, $object_id, $updated_fields, $xbox_instance ){
        $after_saving_member = $object_id && isset( $data['post_type'] ) && $data['post_type'] == $this->plugin->arg( 'post_types', 'members' );
        if( $after_saving_member ){
            $member_id = strval( $object_id );
            $all_teams = Teams::get_saved_teams( $this->plugin->post_types['teams'] );
            $active_teams_ids = isset( $data['tgr_teams'] ) ? $data['tgr_teams'] : array();
            foreach( $all_teams as $team_id => $team ){
                $members = $team->option( 'members' );
                $select_members = $team->get_selected_members_ids();
                if( in_array( $team_id, $active_teams_ids ) ){
                    if( ! in_array( $member_id, $select_members ) ){
                        //Not exists, add member id
                        $select_members[] = $member_id;
                        $members = $members . ',' . $member_id;
                        update_post_meta( $team_id, 'tgr_select-members', $select_members );
                        update_post_meta( $team_id, 'tgr_members', $members );
                    }
                } else{
                    if( in_array( $member_id, $select_members ) ){
                        //Exists, remove member id
                        if( ( $key = array_search( $member_id, $select_members ) ) !== false ){
                            unset( $select_members[$key] );
                        }
                        $members = explode( ',', $members );
                        if( ( $key = array_search( $member_id, $members ) ) !== false ){
                            unset( $members[$key] );
                        }
                        $members = implode( ',', $members );
                        update_post_meta( $team_id, 'tgr_select-members', $select_members );
                        update_post_meta( $team_id, 'tgr_members', $members );
                    }
                }
            }
        }

        //$after_saving_team = $object_id && isset( $data['post_type'] ) && $data['post_type'] == $this->plugin->arg( 'post_types', 'teams' );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if the member is valid
    |---------------------------------------------------------------------------------------------------
    */
    public function is_valid_member( $id = 0 ){
        $member = get_post( $id );
        if( $member && $member->post_type == $this->post_types['members'] ){
            return true;
        }
        return false;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if the team is valid
    |---------------------------------------------------------------------------------------------------
    */
    public function is_valid_team( $id = 0 ){
        $team = get_post( $id );
        if( $team && $team->post_type == $this->post_types['teams'] ){
            return true;
        }
        return false;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if the member is published
    |---------------------------------------------------------------------------------------------------
    */
    public function is_published_member( $id = 0 ){
        return $this->is_valid_member( $id ) && get_post_status( $id ) == 'publish';
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Check if the team is published
    |---------------------------------------------------------------------------------------------------
    */
    public function is_published_team( $id = 0 ){
        return $this->is_valid_team( $id ) && get_post_status( $id ) == 'publish';
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add Duplicate link to Row actions
    |---------------------------------------------------------------------------------------------------
    */
    public function add_duplicate_links( $actions, $post ){
        if( $this->is_published_member( $post->ID ) ){
            $actions['duplicate_member'] = '<a href="#" class="tgr-duplicate-member" data-member_id="' . esc_attr( $post->ID ) . '">' . esc_html__( 'Duplicate Member', 'teamgrid' ) . '</a>';
        }
        if( $this->is_published_team( $post->ID ) ){
            $actions['duplicate_team'] = '<a href="#" class="tgr-duplicate-team" data-team_id="' . esc_attr( $post->ID ) . '">' . esc_html__( 'Duplicate Team', 'teamgrid' ) . '</a>';
        }
        return $actions;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Columns for the member list
    |---------------------------------------------------------------------------------------------------
    */
    public function set_columns_members( $columns ){
        $columns = array(
            "cb" => "<input type=\"checkbox\" />",
            "title" => esc_html__( 'Title', 'teamgrid' ),
            "image" => esc_html__( 'Image', 'teamgrid' ),
            "shortcode" => esc_html__( 'Shortcode', 'teamgrid' ),
            "role" => esc_html__( 'Role', 'teamgrid' ),
            "social" => esc_html__( 'Social Icons', 'teamgrid' ),
            "filters" => esc_html__( 'Filters', 'teamgrid' ),
            "fields" => esc_html__( 'Fields', 'teamgrid' ),
            "skills" => esc_html__( 'Skills', 'teamgrid' ),
            "date" => esc_html__( 'Date' ),
        );
        return $columns;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Content for member list columns
    |---------------------------------------------------------------------------------------------------
    */
    public function set_content_columns_members( $column, $post_id ){
        $member = Members::get( $post_id );
        if( get_post_status( $post_id ) == 'publish' ){
            switch( $column ){
                case 'image':
                    echo wp_kses_post( "<img src='{$member->get_thumbnail()}'>" );
                    break;

                case 'shortcode':
                    $shortcode = '[teamgrid_member id="' . $post_id . '"]';
                    echo wp_kses( "<input type='text' class='tgr-input-shortcode' value='$shortcode' onfocus='this.select()' readonly>", array(
                        'input' => array(
                            'type' => array(),
                            'class' => array(),
                            'value' => array(),
                            'onfocus' => array(),
                            'readonly' => array(),
                        )
                    ) );
                    break;

                case 'role':
                    echo esc_html( $member->option( 'role' ) );
                    break;

                case 'social':
                    echo wp_kses_post( $member->build_item_social() );
                    break;

                case 'filters':
                    echo wp_kses_post( $member->get_member_filters_string( ',<br>' ) );
                    break;

                case 'fields':
                    echo wp_kses_post( $member->get_member_fields_string( '<br>' ) );
                    break;

                case 'skills':
                    echo wp_kses_post( $member->get_member_skills_string( '<br>' ) );
                    break;
            }
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Columns for the team list
    |---------------------------------------------------------------------------------------------------
    */
    public function set_columns_teams( $columns ){
        $columns = array(
            "cb" => "<input type=\"checkbox\" />",
            "title" => esc_html__( 'Title', 'teamgrid' ),
            "shortcode" => esc_html__( 'Shortcode', 'teamgrid' ),
            "layout" => esc_html__( 'Layout', 'teamgrid' ),
            "displayer" => esc_html__( 'Viewer', 'teamgrid' ),
            "columns" => esc_html__( 'Columns', 'teamgrid' ),
            "show-filters" => esc_html__( 'Show filters', 'teamgrid' ),
            "date" => esc_html__( 'Date' ),
        );
        return $columns;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Content for the columns of the teams list
    |---------------------------------------------------------------------------------------------------
    */
    public function set_content_columns_teams( $column, $post_id ){
        $team = Teams::get( $post_id );
        if( get_post_status( $post_id ) == 'publish' ){
            switch( $column ){
                case 'image':
                    echo wp_kses_post( "<img src='{$team->get_thumbnail()}'>" );
                    break;

                case 'shortcode':
                    $shortcode = '[teamgrid_team id="' . $post_id . '"]';
                    echo wp_kses( "<input type='text' class='tgr-input-shortcode' value='$shortcode' onfocus='this.select()' readonly>", array(
                        'input' => array(
                            'type' => array(),
                            'class' => array(),
                            'value' => array(),
                            'onfocus' => array(),
                            'readonly' => array(),
                        )
                    ) );
                    break;

                case 'layout':
                    echo esc_html( $team->layout_number );
                    break;

                case 'displayer':
                    echo esc_html( $team->option( 'displayer' ) );
                    break;

                case 'columns':
                    echo esc_html( $team->option( 'layout-columns' ) );
                    break;

                case 'show-filters':
                    echo esc_html( $team->option( 'filters-show' ) );
                    break;
            }
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Show plugin activation message
    |---------------------------------------------------------------------------------------------------
    */
    public function show_message_to_activate_plugin(){
        $header = esc_html__( 'License activation required.', 'teamgrid' );
        $message = sprintf( esc_html__( 'Please activate your license from %shere%s. Plugin Settings > Plugin Activation', 'teamgrid' ), '<a href="' . esc_url( $this->settings_url ) . '" target="_blank">', '</a>' );
        if( ! Settings::plugin_status() ){
            echo "<div class='notice notice-warning'><p><strong>".$this->arg( 'name' ).": $header</strong> $message</p></div>";
        }
    }
}