<?php namespace TeamGrid\Includes;


class Teams {
    protected static $instance = null;
    public $plugin;
    public static $teams = array();
    public static $teams_loaded = false;
    public static $resources_loaded_in_footer = false;
    public static $resources_loaded_in_header = false;

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    public function __construct( $options = array() ){
        $this->plugin = Functions::get_plugin_instance();

        $this->hooks();
        $this->add_teams();
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Singleton
    |---------------------------------------------------------------------------------------------------
    */
    private function __clone(){
    }//Stopping Clonning of Object

    public function __wakeup(){
    }//Stopping unserialize of object

    public static function init( $plugin = null ){
        if( null === self::$instance ){
            self::$instance = new self( $plugin );
        }
        return self::$instance;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Hooks
    |---------------------------------------------------------------------------------------------------
    */
    private function hooks(){
        add_action( 'wp_head', array( $this, 'load_resources_in_header' ), 10 );
        add_action( 'admin_head', array( $this, 'load_resources_in_header' ), 10 );
        add_action( 'wp_footer', array( $this, 'load_resources_in_footer' ), 10 );
        add_action( 'admin_footer', array( $this, 'load_resources_in_footer' ), 10 );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add a Team instance
    |---------------------------------------------------------------------------------------------------
    */
    public static function add( $team ){
        if( is_a( $team, 'TeamGrid\Includes\Team' ) ){
            self::$teams[$team->id] = $team;
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Gets an instance of Team
    |---------------------------------------------------------------------------------------------------
    */
    public static function get( $team_id = 0 ){
        $plugin = Functions::get_plugin_instance();
        if( ! $plugin->is_published_team( $team_id ) ){
            return null;
        }
        if( ! isset( self::$teams[$team_id] ) ){
            $team = new Team( array(
                'id' => $team_id
            ) );
            self::add( $team );
            return $team;
        }
        return self::$teams[$team_id];
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Gets all created teams
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_all_teams(){
        return self::$teams;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Add all teams
    |---------------------------------------------------------------------------------------------------
    */
    public function add_teams(){
        if( is_admin() && ! Functions::is_editing_post_type( $this->plugin->post_types['teams'] ) ){
            return;
        }
        $teams = self::get_saved_teams( $this->plugin->post_types['teams'] );
        foreach( $teams as $team ){
            self::add( $team );
        }
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns all teams in the database
    |---------------------------------------------------------------------------------------------------
    */
    public static function get_saved_teams( $post_type ){
        $posts = get_posts( array(
            'post_type' => $post_type,
            'post_status' => 'publish',
            'posts_per_page' => -1
        ) );
        $teams = array();
        foreach( $posts as $post ){
            $team = new Team( array(
                'id' => $post->ID,
                'publish_date' => $post->post_date,
            ) );
            $teams[$team->id] = $team;
        }
        return $teams;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Returns all teams in the database
    |---------------------------------------------------------------------------------------------------
    */
    public static function find_teams_by_member_id( $member_id = '' ){
        $member_id = $member_id ? strval( $member_id ) : '';
        if( ! $member_id ) return array();
        if( empty( self::$teams ) ){
            $plugin = Functions::get_plugin_instance();
            self::$teams = self::get_saved_teams( $plugin->post_types['teams'] );
        }
        $teams = self::$teams;
        $find_teams = array();

        foreach( $teams as $team ){
            $select_members = $team->get_selected_members_ids();
            if( in_array( $member_id, $select_members ) ){
                $find_teams[] = $team->id;
            }
        }
        return $find_teams;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Loading resources in header
    |---------------------------------------------------------------------------------------------------
    */
    public function load_resources_in_header(){
        if( self::$resources_loaded_in_header ){
            return;
        }
        self::$resources_loaded_in_header = true;

        echo "\n\n<!-- TGR:TeamGrid:Header:Teams -->";
        echo "\n<script type='text/javascript'>";
        echo "\n var TGR_TEAMS = {};";
        echo "\n</script>";
        foreach( self::$teams as $team_id => $team ){
            /*
            * The setting below is not sanitized. In order to be able to take advantage of this,
             * a user would have to gain access to the database,
             * in which case this is the least of your worries.
             */
            echo "\n" . $team->get_link_google_fonts();
            echo "\n" . $team->build_style();
            echo "\n" . $team->build_custom_style();
            echo "\n" . $team->build_custom_script();
        }
        echo "\n";
        echo "\n<!-- TGR:TeamGrid:Header:Teams End -->\n\n";
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | Loading resources in footer
    |---------------------------------------------------------------------------------------------------
    */
    public function load_resources_in_footer(){
        if( self::$resources_loaded_in_footer ){
            return;
        }
        self::$resources_loaded_in_footer = true;

        $team_js_options = "";
        foreach( self::$teams as $team_id => $team ){
            $team_js_options .= $team->get_js_options();
        }

        echo "\n\n<!-- TGR:TeamGrid:Footer:Teams -->";
        echo "\n<script type='text/javascript'>";
        echo "\n var TGR_TEAMS = TGR_TEAMS || {};";
        /*
         * The setting below is not sanitized. In order to be able to take advantage of this,
		 * a user would have to gain access to the database,
		 * in which case this is the least of your worries.
         */
        echo $team_js_options;

        echo "\n</script>";
        echo "\n<!-- TGR:TeamGrid:Footer:Teams End -->\n\n";
    }
}